package cn.com.qmth.examcloud.core.basic.service.impl;

import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnum;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnumManager;
import cn.com.qmth.examcloud.core.basic.dao.ExamSiteRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgPropertyRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgPropertyEntity;
import cn.com.qmth.examcloud.core.basic.dao.enums.OrgProperty;
import cn.com.qmth.examcloud.core.basic.service.OrgService;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgInfo;

@Service
public class OrgServiceImpl implements OrgService {

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	UserRepo userRepo;

	@Autowired
	ExamSiteRepo examSiteRepo;

	@Autowired
	UserRoleRelationRepo userRoleRelationRepo;

	@Autowired
	OrgPropertyRepo orgPropertyRepo;

	/*
	 * 保存顶级机构
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.basic.service.OrgService#saveRootOrg(cn.com.
	 * qmth.examcloud.core.basic.service.bean.OrgInfo)
	 */
	@Override
	public OrgEntity saveRootOrg(OrgInfo orgInfo) {

		String code = orgInfo.getCode();
		String name = orgInfo.getName();
		Boolean enable = orgInfo.getEnable();
		String contacts = orgInfo.getContacts();
		String telephone = orgInfo.getTelephone();
		String domainName = orgInfo.getDomainName();
		String remark = orgInfo.getRemark();
		Map<String, String> properties = orgInfo.getProperties();

		if (StringUtils.isBlank(code)) {
			throw new StatusException("B-150001", "code is null");
		}
		if (StringUtils.isBlank(name)) {
			throw new StatusException("B-150002", "name is null");
		}

		if (null == enable) {
			enable = true;
		}

		OrgEntity orgEntity = orgRepo.findByParentIdIsNullAndCode(code);
		if (null == orgEntity) {
			OrgEntity tmp = new OrgEntity();
			tmp.setRootId(-1L);
			tmp.setCode(code);
			tmp.setEnable(enable);
			tmp.setName(name);
			orgEntity = orgRepo.save(tmp);
		} else {
			if (!orgEntity.getId().equals(orgEntity.getRootId())) {
				throw new StatusException("B-150003", "数据错误");
			}
		}

		orgEntity.setRootId(orgEntity.getId());
		orgEntity.setName(name);
		orgEntity.setEnable(enable);
		orgEntity.setContacts(contacts);
		orgEntity.setTelephone(telephone);
		orgEntity.setDomainName(domainName);
		orgEntity.setRemark(remark);

		OrgEntity saved = orgRepo.save(orgEntity);

		Map<DynamicEnum, String> map = checkAndGetOrgProperties(properties);
		for (Entry<DynamicEnum, String> entry : map.entrySet()) {
			DynamicEnum de = entry.getKey();
			String value = entry.getValue();
			OrgPropertyEntity entity = orgPropertyRepo.findByOrgIdAndKeyId(saved.getId(),
					de.getId());
			if (null == entity) {
				entity = new OrgPropertyEntity();
				entity.setOrgId(saved.getId());
				entity.setKeyId(de.getId());
			}
			entity.setValue(value);

			orgPropertyRepo.save(entity);
		}

		return saved;
	}

	/*
	 * 保存子机构
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.basic.service.OrgService#saveSubOrg(cn.com.
	 * qmth.examcloud.core.basic.service.bean.OrgInfo)
	 */
	@Override
	public OrgEntity saveSubOrg(OrgInfo orgInfo) {

		String code = orgInfo.getCode();
		String name = orgInfo.getName();
		Boolean enable = orgInfo.getEnable();
		String contacts = orgInfo.getContacts();
		String telephone = orgInfo.getTelephone();
		Long rootId = orgInfo.getRootId();
		Long parentId = orgInfo.getParentId();
		String remark = orgInfo.getRemark();
		Map<String, String> properties = orgInfo.getProperties();

		if (null == enable) {
			enable = true;
		}

		if (null == rootId) {
			throw new StatusException("B-150000", "rootId is null");
		}
		if (StringUtils.isBlank(code)) {
			throw new StatusException("B-150001", "code is null");
		}
		if (StringUtils.isBlank(name)) {
			throw new StatusException("B-150002", "name is null");
		}
		if (null == parentId) {
			throw new StatusException("B-150003", "parentId is null");
		}

		OrgEntity rootOrgEntity = orgRepo.findOne(rootId);
		if (null == rootOrgEntity) {
			throw new StatusException("B-150004", "rootId is wrong");
		}
		if (null != rootOrgEntity.getParentId()) {
			throw new StatusException("B-150005", "rootId is wrong");
		}

		OrgEntity parentOrgEntity = orgRepo.findOne(parentId);
		if (null == parentOrgEntity) {
			throw new StatusException("B-150006", "parentId is wrong");
		}

		if (!rootOrgEntity.getRootId().equals(parentOrgEntity.getRootId())) {
			throw new StatusException("B-150007", "parentId, rootId is wrong");
		}

		OrgEntity orgEntity = orgRepo.findByRootIdAndCode(rootId, code);
		if (null == orgEntity) {
			orgEntity = new OrgEntity();
			orgEntity.setRootId(rootId);
			orgEntity.setParentId(parentId);
		}

		orgEntity.setName(name);
		orgEntity.setEnable(enable);
		orgEntity.setContacts(contacts);
		orgEntity.setTelephone(telephone);
		orgEntity.setRemark(remark);

		OrgEntity saved = orgRepo.save(orgEntity);

		Map<DynamicEnum, String> map = checkAndGetOrgProperties(properties);
		for (Entry<DynamicEnum, String> entry : map.entrySet()) {
			DynamicEnum de = entry.getKey();
			String value = entry.getValue();
			OrgPropertyEntity entity = orgPropertyRepo.findByOrgIdAndKeyId(saved.getId(),
					de.getId());
			if (null == entity) {
				entity = new OrgPropertyEntity();
				entity.setOrgId(saved.getId());
				entity.setKeyId(de.getId());
			}
			entity.setValue(value);

			orgPropertyRepo.save(entity);
		}

		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param properties
	 * @return
	 */
	private Map<DynamicEnum, String> checkAndGetOrgProperties(Map<String, String> properties) {
		DynamicEnumManager manager = OrgProperty.getDynamicEnumManager();

		Map<DynamicEnum, String> map = Maps.newHashMap();
		if (null == properties) {
			return map;
		}
		for (Entry<String, String> entry : properties.entrySet()) {
			String key = entry.getKey();
			String value = entry.getValue();
			if (StringUtils.isBlank(value)) {
				continue;
			}
			DynamicEnum de = null;
			try {
				de = manager.getByName(key);
			} catch (Exception e) {
				throw new StatusException("E-001004", "机构属性错误");
			}
			map.put(de, value.trim());
		}

		return map;
	}

}
